#!/usr/bin/env python3
#
#  arch.buildtree.py - Get Arch PKGBUILD tree
#
import argparse
import os
import shutil
import subprocess
from pathlib import Path

processed = []
pkg = ""
def setup_options():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--tree-dir",
        default="/media/Repo/Trees",
        help="dir where the trees are created"
    )
    opts = parser.parse_args()
    return opts


def git_pull_in_dir(path_to_dir, git):
    original_dir = os.getcwd()
    os.chdir(path_to_dir)
    print(f"Updating {git}")
    exit_code = os.system('git pull 2> /dev/null')
    if exit_code:
        print(f"git update on {git} failed")
    else:
        processed.append(pkg)
    os.chdir(original_dir)
    return exit_code


def git_clone_in_dir(path_to_dir, git):
    original_dir = os.getcwd()
    os.chdir(path_to_dir)
    print(f"Cloning {git}")
    exit_code = os.system("git clone --depth=1 " + git + "  2> /dev/null")
    if exit_code:
        print(f"git clone on {git} failed")
    os.chdir(original_dir)
    return exit_code


def search_and_copy(dir_path, subdir_name):
    for subdir in os.listdir(dir_path):
        if os.path.isdir(os.path.join(dir_path, subdir)) and subdir == subdir_name:
            if os.path.exists(os.path.join(repo_dir, subdir)):
                shutil.rmtree(os.path.join(repo_dir, subdir))
            shutil.copytree(os.path.join(dir_path, subdir), os.path.join(repo_dir, subdir))
            processed.append(subdir_name)
            return True
    return False


def get_base_pkg():
    try:
        output = subprocess.check_output(["expac", "-S1", "%e", pkg], universal_newlines=True)
    except:
        return False
    if output:
        pkgbase = output.strip()
        print(f"Split package, pkgbase is: {pkgbase}")
    else:
        pkgbase = False
    return pkgbase


options = setup_options()

dist = "arch"
repos = ["core", "extra", "community", "multilib"]
svn = "svn"
t_dir = options.tree_dir
if not os.path.exists(t_dir):
    os.makedirs(t_dir)

d_dir = os.path.join(Path(t_dir, dist))
if not os.path.exists(d_dir):
    os.makedirs(d_dir)
svn_dir = os.path.join(Path(d_dir, svn))
if not os.path.exists(svn_dir):
    os.makedirs(svn_dir)

for repo in repos:

    repo_dir = os.path.join(Path(d_dir, repo))
    if not os.path.exists(os.path.join(Path(t_dir, dist, repo))):
        os.makedirs(os.path.join(Path(t_dir, dist, repo)))

# debug = "libsigc++-3.0-docs"
# skipped = False
if os.path.exists(svn_dir + "/svntogit-packages"):
    git_pull_in_dir(svn_dir + "/svntogit-packages", "packages")
else:
    git_clone_in_dir(svn_dir, "https://github.com/archlinux/svntogit-packages.git")

if os.path.exists(svn_dir + "/svntogit-community"):
    git_pull_in_dir(svn_dir + "/svntogit-community", "community")
else:
    git_clone_in_dir(svn_dir, "https://github.com/archlinux/svntogit-community.git")

for repo in repos:
    processed = []
    if repo == "core" or repo == "extra":
        raw_dir = svn_dir + "/svntogit-packages"
    else:
        raw_dir = svn_dir + "/svntogit-community"
    repo_dir = os.path.join(Path(t_dir), dist, repo)
    pkg_list = os.popen("pacman -Sl " + repo + "| awk '{print $2}'").read().splitlines()
    fail = []
    for pkg in pkg_list:
        #     if pkg != debug and not skipped:
        #         continue
        #     else:
        #         skipped = True
        print(f"\nProcessing {pkg} from {repo}")
        copied = search_and_copy(raw_dir, pkg)
        if copied:
            continue
        else:
            is_split_pkg = get_base_pkg()
            if is_split_pkg in processed:
                print(f"pkgbase {is_split_pkg} already copied")
            if is_split_pkg:
                if os.path.exists(os.path.join(raw_dir, is_split_pkg)):
                    continue
                else:
                    copied = search_and_copy(raw_dir, is_split_pkg)
                    if copied:
                        continue
        fail.append(pkg)
    if fail:
        print(f"Failed to retrive ", end="")
        print(", ".join(str(x) for x in fail))
